import taos
from taos.tmq import *

conn = taos.connect()

print("init")
conn.execute("drop topic if exists topic_ctb_column")
conn.execute("drop database if exists py_tmq")
conn.execute("create database if not exists py_tmq vgroups 2")
conn.select_db("py_tmq")
conn.execute("create stable if not exists stb1 (ts timestamp, c1 int, c2 float, c3 binary(10)) tags(t1 int)")
conn.execute("create table if not exists tb1 using stb1 tags(1)")
conn.execute("create table if not exists tb2 using stb1 tags(2)")
conn.execute("create table if not exists tb3 using stb1 tags(3)")

print("create topic")
conn.execute("drop topic if exists topic_ctb_column")
conn.execute("create topic if not exists topic_ctb_column as select ts, c1, c2, c3 from stb1")

print("build consumer")
consumer = TaosConsumer('topic_ctb_column', group_id='123')
for msg in consumer:
    for row in msg:
        print(row)


"""
Query example for RestConnection
"""

from taosrest import connect, TaosRestConnection, Result

conn: TaosRestConnection = connect()
result: Result = conn.query("show databases")

num_of_fields = result.field_count
for field in result.fields:
    print(field)

for row in result:
    print(row)

# {'name': 'name', 'type': 8, 'bytes': 32}
# {'name': 'created_time', 'type': 9, 'bytes': 8}
# {'name': 'ntables', 'type': 4, 'bytes': 4}
# {'name': 'vgroups', 'type': 4, 'bytes': 4}
# {'name': 'replica', 'type': 3, 'bytes': 2}
# {'name': 'quorum', 'type': 3, 'bytes': 2}
# {'name': 'days', 'type': 3, 'bytes': 2}
# {'name': 'keep', 'type': 8, 'bytes': 24}
# {'name': 'cache(MB)', 'type': 4, 'bytes': 4}
# {'name': 'blocks', 'type': 4, 'bytes': 4}
# {'name': 'minrows', 'type': 4, 'bytes': 4}
# {'name': 'maxrows', 'type': 4, 'bytes': 4}
# {'name': 'wallevel', 'type': 2, 'bytes': 1}
# {'name': 'fsync', 'type': 4, 'bytes': 4}
# {'name': 'comp', 'type': 2, 'bytes': 1}
# {'name': 'cachelast', 'type': 2, 'bytes': 1}
# {'name': 'precision', 'type': 8, 'bytes': 3}
# {'name': 'update', 'type': 2, 'bytes': 1}
# {'name': 'status', 'type': 8, 'bytes': 10}
# ['test', datetime.datetime(2022, 6, 13, 9, 38, 15, 290000, tzinfo=datetime.timezone(datetime.timedelta(seconds=28800), '+08:00')), 1, 1, 1, 1, 10, '3650', 16, 6, 100, 4096, 1, 3000, 2, 0, 'ms', 0, 'ready']
# ['log', datetime.datetime(2022, 3, 26, 15, 54, 26, 997000, tzinfo=datetime.timezone(datetime.timedelta(seconds=28800), '+08:00')), 274, 1, 1, 1, 10, '30', 1, 3, 100, 4096, 1, 3000, 2, 0, 'us', 0, 'ready']
# ['test2', datetime.datetime(2022, 4, 21, 9, 34, 52, 236000, tzinfo=datetime.timezone(datetime.timedelta(seconds=28800), '+08:00')), 0, 0, 1, 1, 10, '3650', 16, 6, 100, 4096, 1, 3000, 2, 0, 'ms', 0, 'ready']
# ['power', datetime.datetime(2022, 6, 7, 13, 3, 10, 198000, tzinfo=datetime.timezone(datetime.timedelta(seconds=28800), '+08:00')), 4, 1, 1, 1, 10, '3650', 16, 6, 100, 4096, 1, 3000, 2, 0, 'ms', 0, 'ready']



import taos


def test_invalid_dbname():
    conn: taos.TaosConnection = None
    try:
        conn = taos.connect(database="notexist")
    except taos.Error as e:
        print("\n=============================")
        print(e)
        print("exception class: ", e.__class__.__name__)  # ConnectionError
        print("error number:", e.errno)  # 65535
        print("error message:", e.msg)  # connect to TDengine failed
    except BaseException as other:
        print("exception occur")
        print(other)
    finally:
        if conn is not None:
            conn.close()



from taos.tmq import Consumer
import taos


def init_tmq_env(db, topic):
    conn = taos.connect()
    conn.execute("drop topic if exists {}".format(topic))
    conn.execute("drop database if exists {}".format(db))
    conn.execute("create database if not exists {}".format(db))
    conn.select_db(db)
    conn.execute("create stable if not exists stb1 (ts timestamp, c1 int, c2 float, c3 binary(10)) tags(t1 int)")
    conn.execute("create table if not exists tb1 using stb1 tags(1)")
    conn.execute("create table if not exists tb2 using stb1 tags(2)")
    conn.execute("create table if not exists tb3 using stb1 tags(3)")
    conn.execute("create topic if not exists {} as select ts, c1, c2, c3 from stb1".format(topic))
    conn.execute("insert into tb1 values (now-2s, 1, 1.0, 'tmq test')")
    conn.execute("insert into tb2 values (now-1s, 2, 2.0, 'tmq test')")
    conn.execute("insert into tb3 values (now, 3, 3.0, 'tmq test')")


if __name__ == '__main__':
    # init env
    init_tmq_env("tmq_test", "tmq_test_topic")
    consumer = Consumer(
        {
            "group.id": "3",
            "td.connect.user": "root",
            "td.connect.pass": "taosdata",
            "msg.with.table.name": "true",
            "enable.auto.commit": "true",
        }
    )
    consumer.subscribe(["tmq_test_topic"])

    while True:
        res = consumer.poll(100)
        if not res:
            continue
        err = res.error()
        if err is not None:
            raise err
        val = res.value()

        for block in val:
            print(block.fetchall())



# encoding:UTF-8

"""Constants in TDengine python
"""

import ctypes, struct


class FieldType(object):
    """TDengine Field Types"""

    # type_code
    C_NULL = 0
    C_BOOL = 1
    C_TINYINT = 2
    C_SMALLINT = 3
    C_INT = 4
    C_BIGINT = 5
    C_FLOAT = 6
    C_DOUBLE = 7
    C_VARCHAR = 8
    C_BINARY = 8
    C_TIMESTAMP = 9
    C_NCHAR = 10
    C_TINYINT_UNSIGNED = 11
    C_SMALLINT_UNSIGNED = 12
    C_INT_UNSIGNED = 13
    C_BIGINT_UNSIGNED = 14
    C_JSON = 15
    # NULL value definition
    # NOTE: These values should change according to C definition in tsdb.h
    C_BOOL_NULL = 0x02
    C_TINYINT_NULL = -128
    C_TINYINT_UNSIGNED_NULL = 255
    C_SMALLINT_NULL = -32768
    C_SMALLINT_UNSIGNED_NULL = 65535
    C_INT_NULL = -2147483648
    C_INT_UNSIGNED_NULL = 4294967295
    C_BIGINT_NULL = -9223372036854775808
    C_BIGINT_UNSIGNED_NULL = 18446744073709551615
    C_FLOAT_NULL = ctypes.c_float(struct.unpack("<f", b"\x00\x00\xf0\x7f")[0])
    C_DOUBLE_NULL = ctypes.c_double(struct.unpack("<d", b"\x00\x00\x00\x00\x00\xff\xff\x7f")[0])
    C_BINARY_NULL = bytearray([int("0xff", 16)])
    # Timestamp precision definition
    C_TIMESTAMP_MILLI = 0
    C_TIMESTAMP_MICRO = 1
    C_TIMESTAMP_NANO = 2
    C_TIMESTAMP_UNKNOWN = 3



import json
import os
from urllib.request import urlopen, Request

import pytest
from dotenv import load_dotenv

from decorators import check_env

load_dotenv()

default_token = "/KfeAzX/f9na8qdtNZmtONryp201ma04bEl8LcvLUd7a8qdtNZmtONryp201ma04"


def test_login():
    response = urlopen("http://localhost:6041/rest/login/root/taosdata")
    resp = json.load(response)
    print()
    print(resp)
    assert "code" in resp and resp["code"] == 0
    assert "desc" in resp and resp["desc"] == default_token


def test_wrong_password():
    response = urlopen("http://localhost:6041/rest/login/root/taosdatax")
    resp = json.load(response)
    assert resp["code"] == 3
    assert resp["desc"] == "Authentication failure"


@pytest.mark.skip(reason="know bug TD-16959")
@check_env
def test_server_version():
    url = "http://localhost:6041/rest/sql"
    data = "select server_version()".encode("ascii")

    headers = {
        "Authorization": "Taosd " + default_token
    }
    request = Request(url, data, headers)
    response = urlopen(request)
    resp = json.load(response)
    print(resp)
    assert resp["rows"] == 1
    assert len(resp["column_meta"]) == 1
    assert len(resp["data"]) == 1


@check_env
def test_wrong_sql():
    """
    {'code': 9730, 'desc': 'Table does not exist: notable'}
     """
    url = "http://localhost:6041/rest/sql"
    data = "select * from nodb.notable".encode("utf8")

    headers = {
        "Authorization": "Taosd " + default_token
    }
    request = Request(url, data, headers)
    response = urlopen(request)
    resp = json.load(response)
    print("\n", resp)
    assert "code" in resp and resp["code"] != 0



import pandas
from sqlalchemy import create_engine

for driver in ("taos", "taosrest", "taosws"):
    engine = create_engine(f"{driver}://root:taosdata@localhost")
    res = pandas.read_sql("show databases", engine)
    print(res)



from time import sleep
import taos
from taos.tmq import *
import threading


class insertThread(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)

    def run(self):
        print("Start insert data")
        conn = taos.connect()
        conn.select_db("tmq_test")
        conn.execute("insert into tb1 values (now, true,1,1,1,1,1,1,1,1,1,1,1,'1','1')")
        print("Finish insert data")


class subscribeThread(threading.Thread):
    def __init__(self, consumer):
        threading.Thread.__init__(self)
        self.consumer = consumer

    def run(self):
        print("Start Subscribe")
        for msg in self.consumer:
            topic = msg.get_topic_name()
            db = msg.get_db_name()

            assert topic == "topic1"
            assert db == "tmq_test"

            break

        print("Finish Subscribe")


def test_tmq():
    """This test will test TDengine kafka tmq validity"""
    if not IS_V3:
        return
    conn = taos.connect()
    conn.execute("create database if not exists tmq_test")
    conn.select_db("tmq_test")
    conn.execute(
        "create stable if not exists stb1 (ts timestamp, \
        c1 bool, c2 tinyint unsigned, c3 smallint unsigned, c4 int unsigned, \
        c5 bigint unsigned, c6 tinyint, c7 smallint, c8 tinyint, c9 bigint, c10 float, \
        c11 double, c12 timestamp, c13 varchar(8), c14 nchar(8)) tags \
        (t1 bool, t2 tinyint unsigned, t3 smallint unsigned, t4 int unsigned, \
        t5 bigint unsigned, t6 tinyint, t7 smallint, t8 tinyint, t9 bigint, t10 float, \
        t11 double, t12 timestamp, t13 varchar(8), t14 nchar(8))"
    )
    conn.execute("create table if not exists tb1 using stb1 tags (true, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, '1', '1')")
    print("========start create topic")
    conn.execute("drop topic if exists topic1")
    conn.execute(
        "create topic if not exists topic1 as select ts,c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14 from stb1"
    )
    consumer = TaosConsumer("topic1", group_id="123", msg_with_table_name="true")

    print("====== check subscription")
    sub_list = consumer.subscription()
    assert sub_list[0] == "topic1"

    sThread = subscribeThread(consumer)

    iThread = insertThread()

    sThread.start()
    sleep(2)
    iThread.start()

    iThread.join()
    sThread.join()

    print("====== finish test, start clean")
    print("====== unsubscribe topic")
    consumer.unsubscribe()

    # drop database and topic
    conn.execute("drop topic if exists topic1")
    conn.execute("drop database if exists tmq_test")


if __name__ == "__main__":
    test_tmq()



from datetime import datetime

import taos


def test_query():
    """This test will use fetch_block for rows fetching, significantly faster than rows_iter"""
    conn = taos.connect()
    conn.execute("drop database if exists test_query_py")
    conn.execute("create database if not exists test_query_py")
    conn.execute("use test_query_py")
    conn.execute("create table if not exists tb1 (ts timestamp, v int) tags(jt json)")
    n = conn.execute("insert into tn1 using tb1 tags('{\"name\":\"value\"}') values(now, null) (now + 10s, 1)")
    n = conn.execute("insert into tn1 using tb1 tags('{\"name\":\"value\"}') values(now, null) (now + 10s, 1)")
    print("inserted %d rows" % n)
    result = conn.query("select * from tb1")
    fields = result.fields
    for field in fields:
        print("field: %s" % field)

    # test re-consume fields
    flag = 0
    for _ in fields:
        flag += 1
    assert flag == 3

    start = datetime.now()
    for row in result:
        print(row)
        None

    for row in result.rows_iter():
        print(row)
    
    result = conn.query("select * from tb1 limit 1")
    results = result.fetch_all_into_dict()
    print(results)

    end = datetime.now()
    elapsed = end - start
    print("elapsed time: ", elapsed)
    result.close()
    conn.close()

if __name__ == "__main__":
    test_query()



import datetime
import os
import taos

from dotenv import load_dotenv

from decorators import check_env
from taosrest.restclient import RestClient

load_dotenv()


@check_env
def test_auth():
    url = os.environ["TDENGINE_URL"]
    client = RestClient(url, user="root", password="taosdata")
    print(client._taosd_token)


@check_env
def test_show_database():
    url = os.environ["TDENGINE_URL"]
    client = RestClient(url)
    resp = client.sql("show databases")
    print("\n", resp)


@check_env
def test_insert_data():
    url = os.environ["TDENGINE_URL"]
    c = RestClient(url, password="taosdata", database="test")
    c.sql("drop database if exists test")
    c.sql("create database test")
    resp = c.sql("create table tb2 (ts timestamp, c1 int, c2 double, c3 timestamp)")
    print("\n=====================create table resp================")
    print(resp)
    # {'status': 'succ', 'head': ['affected_rows'], 'column_meta': [['affected_rows', 4, 4]], 'rows': 1, 'data': [[0]]}
    resp = c.sql("insert into tb2 values (now, -100, -200.3, now+1m) (now+10s, -101, -340.2423424, now+2m)")
    print("==============insert resp==============")
    print(resp)
    #  {'status': 'succ', 'head': ['affected_rows'], 'column_meta': [['affected_rows', 4, 4]], 'rows': 1, 'data': [[2]]}
    assert resp["rows"] == 1
    if taos.IS_V3:
        assert resp["column_meta"] == [['affected_rows', "INT", 4]]
    else:
        assert resp["column_meta"] == [['affected_rows', 4, 4]]


@check_env
def test_describe_table():
    url = os.environ["TDENGINE_URL"]
    c = RestClient(url)
    try:
        c.sql("describe test.noexits")
        assert False
    except Exception as e:
        print(e)


@check_env
def test_select_data_with_timestamp_type():
    url = os.environ["TDENGINE_URL"]
    c = RestClient(url)
    resp = c.sql("select * from test.tb2")
    print("\n", resp)
    data = resp["data"]
    assert isinstance(data[0][0], datetime.datetime) and data[0][0].tzinfo is None
    assert isinstance(data[0][3], datetime.datetime) and data[0][3].tzinfo is None

@check_env
def test_use_str_timestamp():
    url = os.environ["TDENGINE_URL"]
    c = RestClient(url, convert_timestamp=False)
    resp = c.sql("select * from test.tb2")
    data = resp["data"]
    print(data[0][0], data[0][3])
    assert isinstance(data[0][0], str) and isinstance(data[0][3], str)

if __name__ == "__main__":
    test_show_database()


from taos import *

conn = connect()

dbname = "pytest_taos_stmt"
conn.execute("drop database if exists %s" % dbname)
conn.execute("create database if not exists %s" % dbname)
conn.select_db(dbname)

conn.execute(
    "create table if not exists log(ts timestamp, bo bool, nil tinyint, \
        ti tinyint, si smallint, ii int, bi bigint, tu tinyint unsigned, \
        su smallint unsigned, iu int unsigned, bu bigint unsigned, \
        ff float, dd double, bb binary(100), nn nchar(100), tt timestamp)",
)

stmt = conn.statement("insert into log values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)")

params = new_bind_params(16)
params[0].timestamp(1626861392589)
params[1].bool(True)
params[2].tinyint(None)
params[3].tinyint(2)
params[4].smallint(3)
params[5].int(4)
params[6].bigint(5)
params[7].tinyint_unsigned(6)
params[8].smallint_unsigned(7)
params[9].int_unsigned(8)
params[10].bigint_unsigned(9)
params[11].float(10.1)
params[12].double(10.11)
params[13].binary("hello")
params[14].nchar("stmt")
params[15].timestamp(1626861392589)
stmt.bind_param(params)

params[0].timestamp(1626861392590)
params[15].timestamp(None)
stmt.bind_param(params)
stmt.execute()

assert stmt.affected_rows == 2
# No need to explicitly close, but ok for you
# result.close()

result = conn.query("select * from log")

for row in result:
    print(row)

# No need to explicitly close, but ok for you
# result.close()
# stmt.close()
# conn.close()



import taos

conn = taos.connect(
    host="127.0.0.1", user="root", password="taosdata", database="", config="/etc/taos", timezone="Asia/Shanghai"
)
cursor = conn.cursor()

sql = "drop database if exists db"
cursor.execute(sql)
sql = "create database if not exists db"
cursor.execute(sql)
sql = "create table db.tb(ts timestamp, n int, bin binary(10), nc nchar(10))"
cursor.execute(sql)
sql = "insert into db.tb values (1650000000000, 1, 'abc', '北京')"
cursor.execute(sql)
sql = "insert into db.tb values (1650000000001, null, null, null)"
cursor.execute(sql)
sql = "select * from db.tb"
cursor.execute(sql)

for row in cursor:
    print(row)



from functools import wraps
import os

class NoEnvError(Exception):
    pass

def check_env(func):
    @wraps(func)
    def _func():
        if "TDENGINE_URL" not in os.environ:
            raise NoEnvError("Please set environment variable TDENGINE_URL")
        func()

    return _func



# encoding:UTF-8
from taos import *

conn = connect()

dbname = "pytest_taos_stmt_multi"
conn.execute("drop database if exists %s" % dbname)
conn.execute("create database if not exists %s" % dbname)
conn.select_db(dbname)

conn.execute(
    "create table if not exists log(ts timestamp, bo bool, nil tinyint, \
        ti tinyint, si smallint, ii int, bi bigint, tu tinyint unsigned, \
        su smallint unsigned, iu int unsigned, bu bigint unsigned, \
        ff float, dd double, bb binary(100), nn nchar(100), tt timestamp)",
)

stmt = conn.statement("insert into log values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)")

params = new_multi_binds(16)
params[0].timestamp((1626861392589, 1626861392590, 1626861392591))
params[1].bool((True, None, False))
params[2].tinyint([-128, -128, None]) # -128 is tinyint null
params[3].tinyint([0, 127, None])
params[4].smallint([3, None, 2])
params[5].int([3, 4, None])
params[6].bigint([3, 4, None])
params[7].tinyint_unsigned([3, 4, None])
params[8].smallint_unsigned([3, 4, None])
params[9].int_unsigned([3, 4, None])
params[10].bigint_unsigned([3, 4, None])
params[11].float([3, None, 1])
params[12].double([3, None, 1.2])
params[13].binary(["abc", "dddafadfadfadfadfa", None])
params[14].nchar(["涛思数据", None, "a long string with 中文字符"])
params[15].timestamp([None, None, 1626861392591])
stmt.bind_param_batch(params)
stmt.execute()

assert stmt.affected_rows == 3

result = conn.query("select * from log")
for row in result:
    print(row)



from time import sleep
import taos
from taos.tmq import *
import threading


class insertThread(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)

    def run(self):
        print("Start insert data")
        conn = taos.connect()
        conn.select_db("tmq_test")
        conn.execute("insert into tb1 values (now, true,1,1,1,1,1,1,1,1,1,1,1,'1','1')")
        print("Finish insert data")


class subscribeThread(threading.Thread):
    def __init__(self, tmq):
        threading.Thread.__init__(self)
        self.tmq = tmq

    def run(self):
        print("Start Subscribe")
        while 1:
            res = self.tmq.poll(1000)
            if res:
                topic = res.get_topic_name()
                db = res.get_db_name()

                assert topic == "topic1"
                assert db == "tmq_test"

                break
        print("Finish Subscribe")


class ConsumerThread(threading.Thread):
    def __init__(self, consumer: Consumer):
        threading.Thread.__init__(self)
        self.consumer = consumer

    def run(self) -> None:
        print("Starting subscribe")
        while True:
            res = self.consumer.poll(1000)
            if not res:
                continue
            assert res.error() is None

            topic = res.topic()
            assert topic == "topic1"

            break
        print("Finished Subscribe")


def pre_test_tmq(precision: str):
    conn = taos.connect()
    conn.execute("drop topic if exists topic1")
    conn.execute("drop database if exists tmq_test")
    if len(precision) > 0:
        conn.execute("create database if not exists tmq_test precision '{}'".format(precision))
    else:
        conn.execute("create database if not exists tmq_test")
    conn.select_db("tmq_test")
    conn.execute(
        "create stable if not exists stb1 (ts timestamp, \
        c1 bool, c2 tinyint unsigned, c3 smallint unsigned, c4 int unsigned, \
        c5 bigint unsigned, c6 tinyint, c7 smallint, c8 tinyint, c9 bigint, c10 float, \
        c11 double, c12 timestamp, c13 varchar(8), c14 nchar(8)) tags \
        (t1 bool, t2 tinyint unsigned, t3 smallint unsigned, t4 int unsigned, \
        t5 bigint unsigned, t6 tinyint, t7 smallint, t8 tinyint, t9 bigint, t10 float, \
        t11 double, t12 timestamp, t13 varchar(8), t14 nchar(8))"
    )
    conn.execute("create table if not exists tb1 using stb1 tags (true, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, '1', '1')")
    print("========start create topic")
    conn.execute(
        "create topic if not exists topic1 as select ts,c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14 from stb1"
    )


def after_ter_tmq():
    conn = taos.connect()
    # drop database and topic
    conn.execute("drop topic if exists topic1")
    conn.execute("drop database if exists tmq_test")


def test_tmq():
    """This test will test TDengine tmq api validity"""
    if not IS_V3:
        return

    pre_test_tmq('')

    conf = TaosTmqConf()
    conf.set("group.id", "1")
    conf.set("td.connect.user", "root")
    conf.set("td.connect.pass", "taosdata")
    conf.set("msg.with.table.name", "true")

    tmq = conf.new_consumer()

    print("====== set topic list")
    topic_list = TaosTmqList()
    topic_list.append("topic1")

    print("====== subscribe topic")
    tmq.subscribe(topic_list)

    print("====== check subscription")
    sub_list = tmq.subscription()
    assert sub_list[0] == "topic1"

    sThread = subscribeThread(tmq)
    iThread = insertThread()

    sThread.start()
    sleep(2)
    iThread.start()

    iThread.join()
    sThread.join()

    print("====== finish test, start clean")
    print("====== unsubscribe topic")
    tmq.unsubscribe()

    after_ter_tmq()


def tmq_with_precision(precision="ms"):
    if not IS_V3:
        return

    pre_test_tmq(precision)
    conn = taos.connect()
    conn.select_db("tmq_test")
    conn.execute("create table if not exists tb1 using stb1 tags (true, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, '1', '1')")
    conn.execute("insert into tb1 values (now, true,1,1,1,1,1,1,1,1,1,1,1,'1','1')")

    conf = TaosTmqConf()
    conf.set("group.id", "1")
    conf.set("td.connect.user", "root")
    conf.set("td.connect.pass", "taosdata")
    conf.set("msg.with.table.name", "true")

    tmq = conf.new_consumer()

    topic_list = TaosTmqList()
    topic_list.append("topic1")
    tmq.subscribe(topic_list)

    res = tmq.poll(1000)
    for row in res:
        print(row)
        assert row[0] is not None
        assert row[2] == 1

    tmq.unsubscribe()
    after_ter_tmq()


def test_tmq_with_precision():
    if not IS_V3:
        return
    tmq_with_precision('ms')
    tmq_with_precision('us')
    tmq_with_precision('ns')


def test_consumer_with_precision():
    tmq_consumer_with_precision('ms')
    tmq_consumer_with_precision('us')
    tmq_consumer_with_precision('ns')


def tmq_consumer_with_precision(precision: str):
    if not IS_V3:
        return
    pre_test_tmq(precision)

    consumer = Consumer({
        "group.id": "1",
        "td.connect.user": "root",
        "td.connect.pass": "taosdata",
        "msg.with.table.name": "true"
    })
    consumer.subscribe(["topic1"])

    sThread = ConsumerThread(consumer)
    iThread = insertThread()

    sThread.start()
    sleep(2)
    iThread.start()

    iThread.join()
    sThread.join()

    print("====== finish test, start clean")
    print("====== unsubscribe topic")
    consumer.unsubscribe()

    after_ter_tmq()


if __name__ == "__main__":
    test_tmq()
    test_tmq_with_precision()
    test_consumer_with_precision()



from sqlalchemy.engine import default, reflection

import taosrest


class AlchemyRestConnection:
    threadsafety = 0
    paramstyle = "pyformat"
    Error = taosrest.Error

    def connect(self, **kwargs):
        host = kwargs["host"] if "host" in kwargs else "localhost"
        port = kwargs["port"] if "port" in kwargs else "6041"
        user = kwargs["username"] if "username" in kwargs else "root"
        password = kwargs["password"] if "password" in kwargs else "taosdata"
        database = kwargs["database"] if "database" in kwargs else None
        url = f"http://{host}:{port}"
        return taosrest.connect(url=url, user=user, password=password, database=database)


class TaosRestDialect(default.DefaultDialect):
    name = "taosrest"
    driver = "taosrest"
    supports_native_boolean = True
    implicit_returning = True

    def do_rollback(self, connection):
        pass

    def _get_server_version_info(self, connection):
        return tuple(connection.connection.server_info)

    @classmethod
    def dbapi(cls):
        return AlchemyRestConnection()

    def has_schema(self, connection, schema):
        return False

    def has_table(self, connection, table_name, schema=None):
        try:
            connection.cursor().execute(f"describe {table_name}")
            return True
        except:
            return False

    @reflection.cache
    def get_indexes(self, connection, table_name, schema=None, **kw):
        """
        Gets all indexes
        """
        # no index is supported by TDengine
        return []

    def get_columns(self, connection, table_name, schema=None, **kw):
        try:
            cursor = connection.cursor()
            cursor.execute("describe {}" % table_name)
            return [row[0] for row in cursor.fetchall()]
        except:
            return []



import taos


def test_cursor():
    conn = taos.connect()
    cursor = conn.cursor()
    cursor.execute("DROP DATABASE IF EXISTS test")
    cursor.execute("CREATE DATABASE test")
    cursor.execute("USE test")
    cursor.execute("CREATE STABLE weather(ts TIMESTAMP, temperature FLOAT) TAGS (location INT)")
    cursor.execute(f"INSERT INTO t1 USING weather TAGS(1) VALUES (now, 23.5) (now+100a, 23.5)")
    assert cursor.rowcount == 2
    cursor.execute("SELECT tbname, ts, temperature, location FROM weather LIMIT 1")
    # rowcount can only get correct value after fetching all data
    all_data = cursor.fetchall()
    assert cursor.rowcount == 1
    cursor.close()
    conn.close()



#! encoding=utf-8
import taos


def test_get_table_vgroup_id():
    conn = taos.connect()
    res = conn.query("select server_version()")
    data = res.fetch_all()
    res.close()
    if data[0][0] < '3.0.2.0':
        return

    conn.execute("drop database if exists test_get_table_vgroup_id")
    conn.execute("create database if not exists test_get_table_vgroup_id")
    conn.execute("create stable test_get_table_vgroup_id.meters (ts timestamp, current float, voltage int, phase float)"
                 " tags (location binary(64), groupId int)")
    conn.execute("create table test_get_table_vgroup_id.d0 using test_get_table_vgroup_id.meters "
                 "tags ('California.SanFrancisco', 1)")
    conn.execute("create table test_get_table_vgroup_id.d1 using test_get_table_vgroup_id.meters "
                 "tags ('California.LosAngles', 2)")

    vg_id = conn.get_table_vgroup_id('test_get_table_vgroup_id', 'd0')
    print(vg_id)
    print('\n')
    vg_id = conn.get_table_vgroup_id('test_get_table_vgroup_id', 'd1')
    print(vg_id)
    print('\n')
    conn.close()


if __name__ == '__main__':
    test_get_table_vgroup_id()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['examples', 'taos', 'taosrest', 'tests']

package_data = \
{'': ['*']}

install_requires = \
['iso8601==1.0.2', 'pytz']

extras_require = \
{'ws:python_version >= "3.7" and python_version < "4.0"': ['taos-ws-py>=0.2.0']}

entry_points = \
{'sqlalchemy.dialects': ['taos = taos.sqlalchemy:TaosDialect',
                         'taosrest = taosrest.sqlalchemy:TaosRestDialect',
                         'taosws = taos.sqlalchemy:TaosWsDialect']}

setup_kwargs = {
    'name': 'taospy',
    'version': '2.7.1',
    'description': 'TDengine connector for python',
    'long_description': '# TDengine Connector for Python\n[![codecov](https://codecov.io/gh/taosdata/taos-connector-python/branch/main/graph/badge.svg?token=BDANN3DBXS)](https://codecov.io/gh/taosdata/taos-connector-python)\n\n[TDengine](https://github.com/taosdata/TDengine) connector for Python enables python programs to access TDengine, using an API which is compliant with the Python DB API 2.0 (PEP-249). It contains two modules:\n\n1. The `taos` module. It uses TDengine C client library for client server communications.\n2. The `taosrest` module. It wraps TDengine RESTful API to Python DB API 2.0 (PEP-249). With this module, you are free to install TDengine C client library.\n\n## Install\n\nYou can use `pip` to install the connector from PyPI:\n\n```bash\npip install taospy\n```\n\nOr with git url:\n\n```bash\npip install git+https://github.com/taosdata/taos-connector-python.git\n```\n\n## Source Code\n\n[TDengine](https://github.com/taosdata/TDengine) connector for Python source code is hosted on [GitHub](https://github.com/taosdata/taos-connector-python).\n\n## Examples for `taosws` Module\n\n### Query with PEP-249 API using `taosws`\n\n```python\nimport taosws\n\n# all parameters are optional\nconn = taosws.connect("taosws://root:taosdata@localhost:6041")\ncursor = conn.cursor()\n\ncursor.execute("show databases")\nresults: list[tuple] = cursor.fetchall()\nfor row in results:\n    print(row)\n```\n\n### Query with query method using `taosws`\n\n```python\nfrom taosws import *\n\nconn = connect("taosws://root:taosdata@localhost:6041")\nresult = conn.query("show databases")\n\nnum_of_fields = result.field_count\nfor field in result.fields:\n    print(field)\n\nfor row in result:\n    print(row)\n```\n\n### Read with Pandas using `taosws`\n\n#### Method one\n\n```python\nimport pandas\nimport taosws\n\nconn = taosws.connect("taosws://root:taosdata@localhost:6041")\ndf: pandas.DataFrame = pandas.read_sql("show databases", conn)\ndf\n```\n\n#### Method Two\n\n```python\nimport pandas\nfrom sqlalchemy import create_engine\n\nengine = create_engine("taosws://root:taosdata@localhost:6041")\ndf: pandas.DataFrame = pandas.read_sql("show databases", engine)\ndf\n```\n\n## Examples for `taosrest` Module\n\n### Query with PEP-249 API\n\n```python\nimport taosrest\n\n# all parameters are optional\nconn = taosrest.connect(url="http://localhost:6041",\n                        user="root",\n                        password="taosdata")\ncursor = conn.cursor()\n\ncursor.execute("show databases")\nresults: list[tuple] = cursor.fetchall()\nfor row in results:\n    print(row)\n```\n\n### Query with query method\n\n```python\nfrom taosrest import connect, TaosRestConnection, Result\n\nconn: TaosRestConnection = connect()\nresult: Result = conn.query("show databases")\n\nnum_of_fields = result.field_count\nfor field in result.fields:\n    print(field)\n\nfor row in result:\n    print(row)\n```\n\n### Read with Pandas\n\n#### Method one\n\n```python\nimport pandas\nimport taos\n\nconn = taos.connect()\ndf: pandas.DataFrame = pandas.read_sql("select * from log.logs", conn)\n```\n\n#### Method two\n\n```python\nimport pandas\nimport taosrest\n\nconn = taosrest.connect()\ndf: pandas.DataFrame = pandas.read_sql("select * from log.logs", conn)\n```\n\n#### Method three\n\n```python\nimport pandas\nfrom sqlalchemy import create_engine\n\nengine = create_engine("taosrest://root:taosdata@localhost:6041")\ndf: pandas.DataFrame = pandas.read_sql("select * from log.logs", engine)\n```\n\n## Examples for `taos` Module\n\n### Connect options\n\nSupported config options:\n\n- **config**: TDengine client configuration directory, by default use "/etc/taos/".\n- **host**: TDengine server host, by default use "localhost".\n- **user**: TDengine user name, default is "root".\n- **password**: TDengine user password, default is "taosdata".\n- **database**: Default connection database name, empty if not set.\n- **timezone**: Timezone for timestamp type (which is `datetime` object with tzinfo in python) no matter what the host\'s timezone is.\n\n```python\nimport taos\n\n# 1. with empty options, connect TDengine by default options\n#   that means:\n#     - use /etc/taos/taos.cfg as default configuration file\n#     - use "localhost" if not set in config file\n#     - use "root" as default username\n#     - use "taosdata" as default password\n#     - use 6030 as default port if not set in config file\n#     - use local timezone datetime as timestamp\nconn = taos.connect()\n# 2. with full set options, default db: log, use UTC datetime.\nconn = taos.connect(host=\'localhost\',\n                    user=\'root\',\n                    password=\'taosdata\',\n                    database=\'log\',\n                    config=\'/etc/taos\',\n                    timezone=\'UTC\')\n```\n\nNote that, the datetime formatted string will contain timezone information when timezone set. For example:\n\n```python\n# without timezone(local timezone depends on your machine)\n\'1969-12-31 16:00:00\'\n# with timezone UTC\n\'1969-12-31 16:00:00+00:00\'\n```\n\n### Query with PEP-249 API\n\n```python\nimport taos\n\nconn = taos.connect()\ncursor = conn.cursor()\n\ncursor.execute("show databases")\nresults = cursor.fetchall()\nfor row in results:\n    print(row)\n\ncursor.close()\nconn.close()\n```\n\n### Query with objective API\n\n```python\nimport taos\n\nconn = taos.connect()\nconn.execute("create database if not exists pytest")\n\nresult = conn.query("show databases")\nnum_of_fields = result.field_count\nfor field in result.fields:\n    print(field)\n\nfor row in result:\n    print(row)\n\nresult.close()\nconn.execute("drop database pytest")\nconn.close()\n```\n\n### Query with async API\n\n```python\nfrom taos import *\nfrom ctypes import *\nimport time\n\n\ndef fetch_callback(p_param, p_result, num_of_rows):\n    print("fetched ", num_of_rows, "rows")\n    p = cast(p_param, POINTER(Counter))\n    result = TaosResult(p_result)\n\n    if num_of_rows == 0:\n        print("fetching completed")\n        p.contents.done = True\n        result.close()\n        return\n\n    if num_of_rows < 0:\n        p.contents.done = True\n        result.check_error(num_of_rows)\n        result.close()\n        return None\n\n    for row in result.rows_iter(num_of_rows):\n        # print(row)\n        None\n\n    p.contents.count += result.row_count\n    result.fetch_rows_a(fetch_callback, p_param)\n\n\ndef query_callback(p_param, p_result, code):\n    # type: (c_void_p, c_void_p, c_int) -> None\n    if p_result is None:\n        return\n\n    result = TaosResult(p_result)\n    if code == 0:\n        result.fetch_rows_a(fetch_callback, p_param)\n\n    result.check_error(code)\n\n\nclass Counter(Structure):\n    _fields_ = [("count", c_int), ("done", c_bool)]\n\n    def __str__(self):\n        return "{ count: %d, done: %s }" % (self.count, self.done)\n\n\ndef test_query(conn):\n    # type: (TaosConnection) -> None\n    counter = Counter(count=0)\n    conn.query_a("select * from log.log", query_callback, byref(counter))\n\n    while not counter.done:\n        print("wait query callback")\n        time.sleep(1)\n\n    print(counter)\n    conn.close()\n\n\nif __name__ == "__main__":\n    test_query(connect())\n```\n\n### Statement API - Bind row after row\n\n```python\nfrom taos import *\n\nconn = connect()\n\ndbname = "pytest_taos_stmt"\nconn.execute("drop database if exists %s" % dbname)\nconn.execute("create database if not exists %s" % dbname)\nconn.select_db(dbname)\n\nconn.execute(\n    "create table if not exists log(ts timestamp, bo bool, nil tinyint, \\\n        ti tinyint, si smallint, ii int, bi bigint, tu tinyint unsigned, \\\n        su smallint unsigned, iu int unsigned, bu bigint unsigned, \\\n        ff float, dd double, bb binary(100), nn nchar(100), tt timestamp)",\n)\n\nstmt = conn.statement("insert into log values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)")\n\nparams = new_bind_params(16)\nparams[0].timestamp(1626861392589)\nparams[1].bool(True)\nparams[2].tinyint(None)\nparams[3].tinyint(2)\nparams[4].smallint(3)\nparams[5].int(4)\nparams[6].bigint(5)\nparams[7].tinyint_unsigned(6)\nparams[8].smallint_unsigned(7)\nparams[9].int_unsigned(8)\nparams[10].bigint_unsigned(9)\nparams[11].float(10.1)\nparams[12].double(10.11)\nparams[13].binary("hello")\nparams[14].nchar("stmt")\nparams[15].timestamp(1626861392589)\nstmt.bind_param(params)\n\nparams[0].timestamp(1626861392590)\nparams[15].timestamp(None)\nstmt.bind_param(params)\nstmt.execute()\n\nassert stmt.affected_rows == 2\n\nresult = conn.query("select * from log")\n\nfor row in result:\n    print(row)\n```\n\n### Statement API - Bind multi rows\n\n```python\nfrom taos import *\n\nconn = connect()\n\ndbname = "pytest_taos_stmt"\nconn.execute("drop database if exists %s" % dbname)\nconn.execute("create database if not exists %s" % dbname)\nconn.select_db(dbname)\n\nconn.execute(\n    "create table if not exists log(ts timestamp, bo bool, nil tinyint, \\\n        ti tinyint, si smallint, ii int, bi bigint, tu tinyint unsigned, \\\n        su smallint unsigned, iu int unsigned, bu bigint unsigned, \\\n        ff float, dd double, bb binary(100), nn nchar(100), tt timestamp)",\n)\n\nstmt = conn.statement("insert into log values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)")\n\nparams = new_multi_binds(16)\nparams[0].timestamp((1626861392589, 1626861392590, 1626861392591))\nparams[1].bool((True, None, False))\nparams[2].tinyint([-128, -128, None])  # -128 is tinyint null\nparams[3].tinyint([0, 127, None])\nparams[4].smallint([3, None, 2])\nparams[5].int([3, 4, None])\nparams[6].bigint([3, 4, None])\nparams[7].tinyint_unsigned([3, 4, None])\nparams[8].smallint_unsigned([3, 4, None])\nparams[9].int_unsigned([3, 4, None])\nparams[10].bigint_unsigned([3, 4, None])\nparams[11].float([3, None, 1])\nparams[12].double([3, None, 1.2])\nparams[13].binary(["abc", "dddafadfadfadfadfa", None])\nparams[14].nchar(["涛思数据", None, "a long string with 中文字符"])\nparams[15].timestamp([None, None, 1626861392591])\nstmt.bind_param_batch(params)\nstmt.execute()\n\nassert stmt.affected_rows == 3\n\nresult = conn.query("select * from log")\nfor row in result:\n    print(row)\n```\n\n### Subscription\n\n```python\nimport taos\nimport random\n\nconn = taos.connect()\ndbname = "pytest_taos_subscribe"\nconn.execute("drop database if exists %s" % dbname)\nconn.execute("create database if not exists %s" % dbname)\nconn.select_db(dbname)\nconn.execute("create table if not exists log(ts timestamp, n int)")\nfor i in range(10):\n    conn.execute("insert into log values(now, %d)" % i)\n\nsub = conn.subscribe(False, "test", "select * from log", 1000)\nprint("# consume from begin")\nfor ts, n in sub.consume():\n    print(ts, n)\n\nprint("# consume new data")\nfor i in range(5):\n    conn.execute("insert into log values(now, %d)(now+1s, %d)" % (i, i))\n    result = sub.consume()\n    for ts, n in result:\n        print(ts, n)\n\nsub.close(True)\nprint("# keep progress consume")\nsub = conn.subscribe(False, "test", "select * from log", 1000)\nresult = sub.consume()\nrows = result.fetch_all()\n# consume from latest subscription needs root privilege(for /var/lib/taos).\nassert result.row_count == 0\nprint("## consumed ", len(rows), "rows")\n\nprint("# consume with a stop condition")\nfor i in range(10):\n    conn.execute("insert into log values(now, %d)" % random.randint(0, 10))\n    result = sub.consume()\n    try:\n        ts, n = next(result)\n        print(ts, n)\n        if n > 5:\n            result.stop_query()\n            print("## stopped")\n            break\n    except StopIteration:\n        continue\n\nsub.close()\n# sub.close()\n\nconn.execute("drop database if exists %s" % dbname)\n# conn.close()\n```\n\n### Subscription asynchronously with callback\n\n```python\nfrom taos import *\nfrom ctypes import *\n\nimport time\n\n\ndef subscribe_callback(p_sub, p_result, p_param, errno):\n    # type: (c_void_p, c_void_p, c_void_p, c_int) -> None\n    print("# fetch in callback")\n    result = TaosResult(c_void_p(p_result))\n    result.check_error(errno)\n    for row in result.rows_iter():\n        ts, n = row()\n        print(ts, n)\n\n\ndef test_subscribe_callback(conn):\n    # type: (TaosConnection) -> None\n    dbname = "pytest_taos_subscribe_callback"\n    try:\n        print("drop if exists")\n        conn.execute("drop database if exists %s" % dbname)\n        print("create database")\n        conn.execute("create database if not exists %s" % dbname)\n        print("create table")\n        # conn.execute("use %s" % dbname)\n        conn.execute("create table if not exists %s.log(ts timestamp, n int)" % dbname)\n\n        print("# subscribe with callback")\n        sub = conn.subscribe(False, "test", "select * from %s.log" % dbname, 1000, subscribe_callback)\n\n        for i in range(10):\n            conn.execute("insert into %s.log values(now, %d)" % (dbname, i))\n            time.sleep(0.7)\n\n        sub.close()\n\n        conn.execute("drop database if exists %s" % dbname)\n        # conn.close()\n    except Exception as err:\n        conn.execute("drop database if exists %s" % dbname)\n        # conn.close()\n        raise err\n\n\nif __name__ == "__main__":\n    test_subscribe_callback(connect())\n```\n\n### Insert with line protocol\n\n```python\nimport taos\nfrom taos import SmlProtocol, SmlPrecision\n\nconn = taos.connect()\ndbname = "pytest_line"\nconn.execute("drop database if exists %s" % dbname)\nconn.execute("create database if not exists %s precision \'us\'" % dbname)\nconn.select_db(dbname)\n\nlines = [\n    \'st,t1=3i64,t2=4f64,t3="t3" c1=3i64,c3=L"pass",c2=false,c4=4f64 1626006833639000000\',\n]\nconn.schemaless_insert(lines, taos.SmlProtocol.LINE_PROTOCOL, taos.SmlPrecision.NOT_CONFIGURED)\nprint("inserted")\n\nconn.schemaless_insert(lines, taos.SmlProtocol.LINE_PROTOCOL, taos.SmlPrecision.NOT_CONFIGURED)\n\nresult = conn.query("show tables")\nfor row in result:\n    print(row)\n\nconn.execute("drop database if exists %s" % dbname)\n```\n\n### Read with Pandas\n\n#### Method one\n\n```python\nimport pandas\nimport taos\n\nconn = taos.connect()\ndf: pandas.DataFrame = pandas.read_sql("select * from log.logs", conn)\n```\n\n#### Method Two\n\n```python\nimport pandas\nfrom sqlalchemy import create_engine\n\nengine = create_engine("taos://root:taosdata@localhost:6030/log")\ndf: pandas.DataFrame = pandas.read_sql("select * from logs", engine)\n```\n\n## Limitation\n\n- `taosrest` is designed to use with taosAdapter. If your TDengine version is older than v2.4.0.0, taosAdapter may not be available.\n\n## License\n\nWe use MIT license for Python connector.\n',
    'author': 'Taosdata Inc.',
    'author_email': 'support@taosdata.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import taosrest
import os


def connect_to_cloud_use_token(url, token):
    conn = taosrest.connect(url=url,
                            token=token)

    print(conn.server_info)
    cursor = conn.cursor()
    cursor.execute("drop database if exists pytest")
    cursor.execute("create database pytest precision 'ns' keep 365")
    cursor.execute("create table pytest.temperature(ts timestamp, temp int)")
    cursor.execute("insert into pytest.temperature values(now, 1) (now+10b, 2)")
    cursor.execute("select * from pytest.temperature")
    rows = cursor.fetchall()
    print(rows)


if __name__ == '__main__':
    url = os.environ["TDENGINE_CLOUD_URL"]
    token = os.environ["TDENGINE_CLOUD_TOKEN"]
    connect_to_cloud_use_token(url, token)



import taosrest

# all parameters are optional
conn = taosrest.connect(url="http://localhost:6041",
                        user="root",
                        password="taosdata")
cursor = conn.cursor()

cursor.execute("show databases")
results: list[tuple] = cursor.fetchall()
for row in results:
    print(row)


import datetime

import taosrest
import pytest
import os
from decorators import check_env
from dotenv import load_dotenv

load_dotenv()


@check_env
def test_fetch_all():
    url = os.environ["TDENGINE_URL"]
    conn = taosrest.connect(url=url,
                            password="taosdata")
    cursor = conn.cursor()

    cursor.execute("show databases")
    results: list[tuple] = cursor.fetchall()
    for row in results:
        print(row)
    print(cursor.description)


@check_env
def test_fetch_one():
    url = os.environ["TDENGINE_URL"]

    conn = taosrest.connect(url=url,
                            user="root",
                            password="taosdata")
    c = conn.cursor()
    c.execute("drop database if exists test")
    c.executemany("create database test")
    c.execute("create table test.tb (ts timestamp, c1 int, c2 double)")
    c.execute("insert into test.tb values (now, -100, -200.3) (now+10s, -101, -340.2423424)")
    assert c.rowcount == 2
    assert c.affected_rows == 2
    c.execute("select * from test.tb")
    assert c.rowcount == 2
    assert c.affected_rows is None
    print()
    row = c.fetchone()
    while row is not None:
        print(row)
        row = c.fetchone()


@check_env
def test_row_count():
    url = os.environ["TDENGINE_URL"]
    conn = taosrest.connect(url=url, user="root", password="taosdata")
    cursor = conn.cursor()
    cursor.execute("select * from test.tb")
    assert cursor.rowcount == 2


@check_env
def test_get_server_info():
    url = os.environ["TDENGINE_URL"]
    conn = taosrest.connect(url=url,
                            user="root",
                            password="taosdata")

    version: str = conn.server_info
    assert len(version.split(".")) == 4


@check_env
def test_execute():
    url = os.environ["TDENGINE_URL"]
    c = taosrest.connect(url=url)
    c.execute("drop database if exists test")
    c.execute("create database test")
    c.execute("create table test.tb (ts timestamp, c1 int, c2 double)")
    affected_rows = c.execute("insert into test.tb values (now, -100, -200.3) (now+10s, -101, -340.2423424)")
    assert affected_rows == 2
    affected_rows = c.execute("select * from test.tb")
    assert affected_rows is None


@check_env
def test_query():
    """
    Note: run it immediately after `test_execute`
    """
    url = os.environ["TDENGINE_URL"]
    c = taosrest.connect(url=url)
    r = c.query("select * from test.tb")
    assert r.rows == 2


@check_env
def test_default_database():
    """
    Note: run it immediately after `test_query`
    """
    url = os.environ["TDENGINE_URL"]
    c = taosrest.connect(url=url, database="test")
    r = c.query("select * from tb")
    assert r.rows == 2


@check_env
def test_no_timezone():
    url = os.environ["TDENGINE_URL"]
    c = taosrest.connect(url=url)
    r = c.query("select * from test.tb")
    for row in r:
        print(row)  # [datetime.datetime(2022, 7, 26, 5, 56, 58, 746000), -100, -200.3]


@check_env
def test_str_timezone():
    url = os.environ["TDENGINE_URL"]
    c = taosrest.connect(url=url, timezone="Asia/Shanghai")
    r = c.query("select * from test.tb")
    for row in r:
        print(row)  # [datetime.datetime(2022, 7, 26, 13, 56, 58, 746000, tzinfo=<DstTzInfo 'Asia/Shanghai' CST+8:00:00 STD>), -100, -200.3]


@check_env
def test_tzinfo_timezone():
    url = os.environ["TDENGINE_URL"]
    tz = datetime.datetime.now().astimezone().tzinfo
    c = taosrest.connect(url=url, timezone=tz)
    r = c.query("select * from test.tb")
    for row in r:
        print(row)  # [datetime.datetime(2022, 7, 26, 13, 56, 58, 746000, tzinfo=datetime.timezone(datetime.timedelta(seconds=28800), 'CST')), -100, -200.3]
